@@ -1,21 +1,17 @@
-import asyncio
 import logging
-import signal
-from asyncio import AbstractEventLoop
 from asyncio import Lock
 from collections import deque
 from collections.abc import Awaitable
+from collections.abc import Callable
 from collections.abc import Iterable
 from typing import Any
-from typing import Callable
-from typing import Optional
-from typing import Union
 
 from aiokafka import AIOKafkaConsumer
 from aiokafka import AIOKafkaProducer
 from aiokafka import ConsumerRecord
 from aiokafka import TopicPartition
 from amgi_common import Lifespan
+from amgi_common import server_serve
 from amgi_common import Stoppable
 from amgi_types import AMGIApplication
 from amgi_types import AMGISendEvent
@@ -30,21 +26,20 @@ logger = logging.getLogger("amgi-aiokafka.error")
 def run(
     app: AMGIApplication,
     *topics: Iterable[str],
-    bootstrap_servers: Union[str, list[str]] = "localhost",
-    group_id: Optional[str] = None,
+    bootstrap_servers: str | list[str] = "localhost",
+    group_id: str | None = None,
 ) -> None:
     server = Server(
         app, *topics, bootstrap_servers=bootstrap_servers, group_id=group_id
     )
-    loop = asyncio.get_event_loop()
-    loop.run_until_complete(_serve(server, loop))
+    server_serve(server)
 
 
 def _run_cli(
     app: AMGIApplication,
     topics: list[str],
-    bootstrap_servers: Optional[list[str]] = None,
-    group_id: Optional[str] = None,
+    bootstrap_servers: list[str] | None = None,
+    group_id: str | None = None,
 ) -> None:
     run(
         app,
@@ -54,40 +49,41 @@ def _run_cli(
     )
 
 
-class _RecordsEvents:
+class _Receive:
     def __init__(
         self,
-        consumer: AIOKafkaConsumer,
         records: Iterable[ConsumerRecord],
-        message_send: Callable[[MessageSendEvent], Awaitable[None]],
-        ackable_consumer: bool,
     ) -> None:
         self._deque = deque(records)
-        self._consumer = consumer
-        self._message_send = message_send
-        self._message_receive_ids: dict[str, dict[TopicPartition, int]] = {}
-        self._ackable_consumer = ackable_consumer
 
-    async def receive(self) -> MessageReceiveEvent:
+    async def __call__(self) -> MessageReceiveEvent:
         record = self._deque.popleft()
-        message_receive_id = f"{record.topic}:{record.partition}:{record.offset}"
-        if self._ackable_consumer:
-            self._message_receive_ids[message_receive_id] = {
-                TopicPartition(record.topic, record.partition): record.offset + 1
-            }
-
         encoded_headers = [(key.encode(), value) for key, value in record.headers]
-        message_receive_event: MessageReceiveEvent = {
+
+        return {
             "type": "message.receive",
-            "id": message_receive_id,
+            "id": f"{record.topic}:{record.partition}:{record.offset}",
             "headers": encoded_headers,
             "payload": record.value,
             "bindings": {"kafka": {"key": record.key}},
             "more_messages": len(self._deque) != 0,
         }
-        return message_receive_event
 
-    async def send(self, event: AMGISendEvent) -> None:
+
+class _Send:
+    def __init__(
+        self,
+        consumer: AIOKafkaConsumer,
+        message_receive_ids: dict[str, dict[TopicPartition, int]],
+        message_send: Callable[[MessageSendEvent], Awaitable[None]],
+        ackable_consumer: bool,
+    ) -> None:
+        self._consumer = consumer
+        self._message_send = message_send
+        self._message_receive_ids = message_receive_ids
+        self._ackable_consumer = ackable_consumer
+
+    async def __call__(self, event: AMGISendEvent) -> None:
         if event["type"] == "message.ack" and self._ackable_consumer:
             offsets = self._message_receive_ids.pop(event["id"])
             await self._consumer.commit(offsets)
@@ -102,15 +98,15 @@ class Server:
         self,
         app: AMGIApplication,
         *topics: Iterable[str],
-        bootstrap_servers: Union[str, list[str]],
-        group_id: Optional[str],
+        bootstrap_servers: str | list[str],
+        group_id: str | None,
     ) -> None:
         self._app = app
         self._topics = topics
         self._bootstrap_servers = bootstrap_servers
         self._group_id = group_id
         self._ackable_consumer = self._group_id is not None
-        self._producer: Optional[AIOKafkaProducer] = None
+        self._producer: AIOKafkaProducer | None = None
         self._producer_lock = Lock()
         self._stoppable = Stoppable()
 
@@ -141,22 +137,30 @@ class Server:
                         "state": state.copy(),
                     }
 
-                    records_events = _RecordsEvents(
-                        self._consumer,
-                        records,
-                        self._message_send,
-                        self._ackable_consumer,
-                    )
+                    message_receive_ids = {
+                        f"{record.topic}:{record.partition}:{record.offset}": {
+                            TopicPartition(
+                                record.topic, record.partition
+                            ): record.offset
+                            + 1
+                        }
+                        for record in records
+                    }
 
                     await self._app(
                         scope,
-                        records_events.receive,
-                        records_events.send,
+                        _Receive(records),
+                        _Send(
+                            self._consumer,
+                            message_receive_ids,
+                            self._message_send,
+                            self._ackable_consumer,
+                        ),
                     )
 
     async def _get_producer(self) -> AIOKafkaProducer:
-        if self._producer is None:
-            async with self._producer_lock:
+        async with self._producer_lock:
+            if self._producer is None:
                 producer = AIOKafkaProducer(bootstrap_servers=self._bootstrap_servers)
                 await producer.start()
                 self._producer = producer
@@ -176,11 +180,3 @@ class Server:
 
     def stop(self) -> None:
         self._stoppable.stop()
-
-
-async def _serve(server: Server, loop: AbstractEventLoop) -> None:
-    signals = (signal.SIGHUP, signal.SIGTERM, signal.SIGINT)
-    for s in signals:
-        loop.add_signal_handler(s, server.stop)
-
-    await server.serve()
